"""Explore Example app Ajax views
"""
import json
import logging

from django.http import HttpResponse
from django.http.response import HttpResponseBadRequest
from django.utils.decorators import method_decorator
from django.utils.html import escape
from django.views.generic import View

import core_main_app.utils.decorators as decorators
from core_main_app.commons import exceptions
from core_main_app.components.template import api as template_api
from core_parser_app.components.data_structure_element import (
    api as data_structure_element_api,
)
from xml_utils.xsd_tree.operations.namespaces import get_namespaces, get_default_prefix
from xml_utils.html_tree import parser as html_tree_parser
from core_explore_common_app.components.query import api as query_api
from core_explore_common_app.views.user.ajax import CreatePersistentQueryUrlView
import core_explore_example_app.permissions.rights as rights


from core_explore_example_app.apps import ExploreExampleAppConfig
from core_explore_example_app.commons.exceptions import MongoQueryException
from core_explore_example_app.components.explore_data_structure import (
    api as explore_data_structure_api,
)
from core_explore_example_app.components.persistent_query_example.models import (
    PersistentQueryExample,
)
from core_explore_example_app.components.saved_query import api as saved_query_api
from core_explore_example_app.components.saved_query.models import SavedQuery
from core_explore_example_app.utils.custom_checkbox_renderer import (
    CustomCheckboxRenderer,
)
from core_explore_example_app.utils.displayed_query import (
    sub_elements_to_pretty_query,
    fields_to_pretty_query,
)
from core_explore_example_app.utils.mongo_query import (
    get_parent_name,
    sub_elements_to_query,
    check_query_form,
)
from core_explore_example_app.utils.parser import remove_form_element, get_parser
from core_explore_example_app.utils.query_builder import (
    render_initial_form,
    render_new_query,
    render_new_criteria,
    render_sub_elements_query,
    prune_html_tree,
    get_user_inputs,
)


logger = logging.getLogger(__name__)


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def generate_element(request, explore_data_structure_id):
    """Generate an element absent from the form.

    Args:
        request:
        explore_data_structure_id:

    Returns:

    """
    try:
        element_id = request.POST["id"]
        explore_data_structure = explore_data_structure_api.get_by_id(
            explore_data_structure_id
        )
        template = template_api.get_by_id(
            str(explore_data_structure.template.id), request=request
        )
        xsd_parser = get_parser(request=request)
        html_form = xsd_parser.generate_element_absent(
            element_id,
            template.content,
            data_structure=explore_data_structure,
            renderer_class=CustomCheckboxRenderer,
        )
    except Exception as exception:
        return HttpResponseBadRequest(
            "An unexpected error occurred: %s" % escape(str(exception)),
            content_type="application/javascript",
        )

    return HttpResponse(html_form)


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def generate_choice(request, explore_data_structure_id):
    """Generate a choice branch absent from the form.

    Args:
        request:
        explore_data_structure_id:

    Returns:

    """
    try:
        element_id = request.POST["id"]
        explore_data_structure = explore_data_structure_api.get_by_id(
            explore_data_structure_id
        )
        template = template_api.get_by_id(
            str(explore_data_structure.template.id), request=request
        )
        xsd_parser = get_parser(request=request)
        html_form = xsd_parser.generate_choice_absent(
            element_id,
            template.content,
            data_structure=explore_data_structure,
            renderer_class=CustomCheckboxRenderer,
        )
    except Exception as exception:
        return HttpResponseBadRequest(
            "An unexpected error occurred: %s" % escape(str(exception)),
            content_type="application/javascript",
        )

    return HttpResponse(html_form)


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def remove_element(request):
    """Remove an element from the form.

    Args:
        request:

    Returns:

    """
    element_id = request.POST["id"]
    code, html_form = remove_form_element(request, element_id)
    return HttpResponse(json.dumps({"code": code, "html": html_form}))


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def save_fields(request):
    """Saves the fields selected by the user

    Args:
        request:

    Returns:

    """
    try:
        # get parameters form request
        form_content = request.POST["formContent"]
        template_id = request.POST["templateID"]

        # modify the form string to only keep the selected elements
        html_tree = html_tree_parser.from_string(form_content)
        any_checked = prune_html_tree(html_tree)

        # get explore data structure
        explore_data_structure = (
            explore_data_structure_api.get_by_user_id_and_template_id(
                str(request.user.id), template_id
            )
        )

        # if checkboxes were checked
        if any_checked:
            # save html form by adding the encoding to unicode, we force to_string
            # method to return a string rather than bytes.
            explore_data_structure.selected_fields_html_tree = (
                html_tree_parser.to_string(html_tree, encoding="unicode")
            )
        else:
            # otherwise, empty any previously saved html form
            explore_data_structure.selected_fields_html_tree = None

        # update explore data structure
        explore_data_structure_api.upsert(explore_data_structure)

        return HttpResponse(json.dumps({}), content_type="application/javascript")
    except Exception as exception:
        logger.error(escape(str(exception)))
        return HttpResponseBadRequest("An error occurred while saving the form.")


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def select_element(request):
    """Select an element in the custom tree

    Args:
        request:

    Returns:

    """
    # get element id
    element_id = request.POST["elementID"]

    # get schema element
    schema_element = data_structure_element_api.get_by_id(element_id, request)

    response_dict = {
        "elementName": schema_element.options["label"],
        "elementID": element_id,
    }

    return HttpResponse(
        json.dumps(response_dict), content_type="application/javascript"
    )


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def get_sub_elements_query_builder(request):
    """Build the form for queries on sub elements

    Args:
        request:

    Returns:

    """
    leaves_id = request.POST["leavesID"]
    template_id = request.POST["templateID"]

    # get list of ids from string
    list_leaves_id = leaves_id.split(" ")

    # get template
    template = template_api.get_by_id(template_id, request=request)

    # get template namespaces
    namespaces = get_namespaces(template.content)
    # get default prefix
    default_prefix = get_default_prefix(namespaces)

    # get the parent name using the first schema element of the list
    parent_name = get_parent_name(list_leaves_id[0], namespaces, request)

    form_fields = []
    for leaf_id in list_leaves_id:
        data_structure_element = data_structure_element_api.get_by_id(leaf_id, request)
        element_type = data_structure_element.options["type"]
        element_name = data_structure_element.options["name"]

        user_inputs = get_user_inputs(
            element_type, data_structure_element, default_prefix
        )

        form_fields.append(
            {
                "element_id": leaf_id,
                "element_name": element_name,
                "element_type": element_type,
                "html": user_inputs,
            }
        )

    response_dict = {
        "subElementQueryBuilder": render_sub_elements_query(parent_name, form_fields)
    }
    return HttpResponse(
        json.dumps(response_dict), content_type="application/javascript"
    )


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def insert_sub_elements_query(request):
    """Inserts a query for a sub element in the query builder

    Args:
        request:

    Returns:

    """
    form_values = json.loads(request.POST["formValues"])
    template_id = request.POST["templateID"]
    criteria_id = request.POST["criteriaID"]

    # get template
    template = template_api.get_by_id(template_id, request=request)

    # get template namespaces
    namespaces = get_namespaces(template.content)
    # get default prefix
    default_prefix = get_default_prefix(namespaces)

    # keep only selected fields
    form_values = [field for field in form_values if field["selected"] is True]
    errors = check_query_form(form_values, template_id, request=request)

    if len(errors) == 0:
        query = sub_elements_to_query(form_values, namespaces, default_prefix, request)
        displayed_query = sub_elements_to_pretty_query(form_values, namespaces, request)
        ui_id = "ui" + criteria_id[4:]
        temporary_query = SavedQuery(
            user_id=ExploreExampleAppConfig.name,
            template=template,
            query=json.dumps(query),
            displayed_query=displayed_query,
        )
        saved_query_api.upsert(temporary_query)
        response_dict = {
            "criteriaID": criteria_id,
            "prettyQuery": displayed_query,
            "uiID": ui_id,
            "queryID": str(temporary_query.id),
        }
    else:
        return HttpResponseBadRequest(
            "<br/>".join(errors), content_type="application/javascript"
        )

    return HttpResponse(
        json.dumps(response_dict), content_type="application/javascript"
    )


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def update_user_input(request):
    """Updates the user input of the query builder according to the type of the
    selected element

    Args:
        request:

    Returns:

    """
    from_element_id = request.POST["elementID"]
    template_id = request.POST["templateID"]

    # get schema element
    data_structure_element = data_structure_element_api.get_by_id(
        from_element_id, request
    )
    # get template
    template = template_api.get_by_id(template_id, request=request)

    # convert xml path to mongo dot notation
    namespaces = get_namespaces(template.content)
    default_prefix = get_default_prefix(namespaces)

    element_type = data_structure_element.options["type"]
    user_inputs = get_user_inputs(element_type, data_structure_element, default_prefix)

    response_dict = {"userInputs": user_inputs, "element_type": element_type}
    return HttpResponse(
        json.dumps(response_dict), content_type="application/javascript"
    )


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def add_criteria(request):
    """Adds an empty criteria to the query builder

    Args:
        request:

    Returns:

    """
    tag_id = request.POST["tagID"]
    new_criteria_html = render_new_criteria(tag_id)

    response_dict = {"criteria": new_criteria_html}
    return HttpResponse(
        json.dumps(response_dict), content_type="application/javascript"
    )


def _render_errors(errors):
    """Renders a list of errors in a paragraph

    Args:
        errors:

    Returns:

    """
    return "<br/>".join(errors)


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def clear_criteria(request):
    """Clears the Query Builder

    Args:
        request:

    Returns:

    """
    new_form = render_initial_form()
    response_dict = {"queryForm": new_form}
    return HttpResponse(
        json.dumps(response_dict), content_type="application/javascript"
    )


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_DELETE_QUERY,
    raise_exception=True,
)
def clear_queries(request):
    """Deletes all saved queries

    Args:
        request:

    Returns:

    """
    template_id = request.POST["templateID"]
    saved_queries = saved_query_api.get_all_by_user_and_template(
        user_id=str(request.user.id), template_id=template_id
    )
    for saved_query in saved_queries:
        saved_query_api.delete(saved_query)

    return HttpResponse(json.dumps({}), content_type="application/javascript")


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_DELETE_QUERY,
    raise_exception=True,
)
def delete_query(request):
    """Deletes a query and update the HTML display

    Args:
        request:

    Returns:

    """
    saved_query_id = request.POST["savedQueryID"]
    try:
        saved_query = saved_query_api.get_by_id(saved_query_id[5:])
    except exceptions.DoesNotExist:
        return HttpResponseBadRequest("The saved query does not exist anymore.")
    saved_query_api.delete(saved_query)
    return HttpResponse(json.dumps({}), content_type="application/javascript")


@decorators.permission_required(
    content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
    permission=rights.EXPLORE_EXAMPLE_ACCESS,
    raise_exception=True,
)
def add_query_criteria(request):
    """Adds the selected query to query builder

    Args:
        request:

    Returns:

    """
    tag_id = request.POST["tagID"]
    saved_query_id = request.POST["savedQueryID"]

    # check if first criteria
    is_first = True if int(tag_id) == 1 else False

    # get saved query number from id
    try:
        saved_query = saved_query_api.get_by_id(saved_query_id)
    except exceptions.DoesNotExist:
        return HttpResponseBadRequest("The saved query does not exist anymore.")

    new_query_html = render_new_query(tag_id, saved_query, is_first)
    response_dict = {"query": new_query_html, "first": is_first}
    return HttpResponse(
        json.dumps(response_dict), content_type="application/javascript"
    )


class GetQueryView(View):
    """Get Query View"""

    fields_to_query_func = None

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
            permission=rights.EXPLORE_EXAMPLE_ACCESS,
            raise_exception=True,
        )
    )
    def post(self, request):
        """Get a query

        Args:
            request:

        Returns:

        """
        try:
            template_id = request.POST["templateID"]
            query_id = request.POST["queryID"]
            form_values = (
                json.loads(request.POST["formValues"])
                if "formValues" in request.POST
                else None
            )
            order_by_field = request.POST["orderByField"].strip()
            order_by_field_array = order_by_field.split(";")

            # save current query builder in session to restore it when coming
            # back to the page
            if "queryForm" in request.POST:
                query_form = request.POST["queryForm"]
                request.session["savedQueryFormExplore"] = query_form

            errors = []
            query_object = query_api.get_by_id(query_id, request.user)
            # set the data-sources sorting value according to the POST request field
            for data_sources_index in range(len(query_object.data_sources)):
                # updating only the existing data-sources (the new data-source already got
                # the default filter value)
                if data_sources_index in range(0, len(order_by_field_array)):
                    query_object.data_sources[data_sources_index][
                        "order_by_field"
                    ] = order_by_field_array[data_sources_index]
            if len(query_object.data_sources) == 0:
                errors.append("Please select at least 1 data source.")

            if len(errors) == 0 and form_values:
                errors.append(
                    check_query_form(form_values, template_id, request=request)
                )
                query_content = self.fields_to_query_func(
                    form_values, template_id, request=request
                )
                query_object.content = json.dumps(query_content)
            elif len(errors) > 0:
                return HttpResponseBadRequest(
                    _render_errors(errors), content_type="application/javascript"
                )

            query_api.upsert(query_object, request.user)

            return HttpResponse(json.dumps({}), content_type="application/javascript")
        except exceptions.ModelError:
            return HttpResponseBadRequest(
                "Invalid input.", content_type="application/javascript"
            )
        except Exception as exception:
            return HttpResponseBadRequest(
                "An unexpected error occurred: %s" % escape(str(exception)),
                content_type="application/javascript",
            )


class SaveQueryView(View):
    """Save Query View"""

    fields_to_query_func = None

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
            permission=rights.EXPLORE_EXAMPLE_SAVE_QUERY,
            raise_exception=True,
        )
    )
    def post(self, request):
        """Save a query and update the html display

        Args:
            request:

        Returns:

        """
        form_values = json.loads(request.POST["formValues"])
        template_id = request.POST["templateID"]

        # Check that the user can save a query
        if "_auth_user_id" not in request.session:
            error = "You have to login to save a query."
            return HttpResponseBadRequest(error, content_type="application/javascript")

        # Check that the query is valid
        errors = check_query_form(form_values, template_id, request=request)
        if len(errors) == 0:
            try:
                query = self.fields_to_query_func(
                    form_values, template_id, request=request
                )
                displayed_query = fields_to_pretty_query(form_values)

                # save the query in the data base
                saved_query = SavedQuery(
                    user_id=str(request.user.id),
                    template=template_api.get_by_id(template_id, request=request),
                    query=json.dumps(query),
                    displayed_query=displayed_query,
                )
                saved_query_api.upsert(saved_query)
            except MongoQueryException as exception:
                errors = [str(exception)]
                return HttpResponseBadRequest(
                    _render_errors(errors), content_type="application/javascript"
                )
        else:
            return HttpResponseBadRequest(
                _render_errors(errors), content_type="application/javascript"
            )

        return HttpResponse(json.dumps({}), content_type="application/javascript")


class CreatePersistentQueryExampleUrlView(CreatePersistentQueryUrlView):
    """Create the persistent url from a Query"""

    view_to_reverse = "core_explore_example_results_redirect"

    @staticmethod
    def _create_persistent_query(query):
        # create the persistent query
        return PersistentQueryExample(
            user_id=query.user_id,
            content=query.content,
            data_sources=query.data_sources,
        )



"""Parser util for explore app
"""

from core_parser_app.components.data_structure_element import (
    api as data_structure_element_api,
)
from core_parser_app.tools.parser.parser import XSDParser, remove_child_element
from core_explore_example_app.settings import PARSER_DOWNLOAD_DEPENDENCIES
from core_explore_example_app.utils.custom_checkbox_renderer import (
    CustomCheckboxRenderer,
)


# TODO: the form renders 'add' buttons based on maxOccurs attributes, but we
#  don't need more than one of each element


def get_parser(request=None):
    """Returns parser

    Returns:

    """

    return XSDParser(
        min_tree=True,
        ignore_modules=True,
        collapse=True,
        auto_key_keyref=False,
        implicit_extension_base=False,
        download_dependencies=PARSER_DOWNLOAD_DEPENDENCIES,
        store_type=True,
        request=request,
    )


# FIXME: refactor common code with core curate app (note: no xml_string here)
def generate_form(xsd_string, data_structure=None, request=None):
    """Generates the form using the parser, returns the root element

    Args:
        xsd_string:
        data_structure:
        request:

    Returns:

    """
    # build parser
    parser = get_parser(request=request)
    # generate form
    root_element_id = parser.generate_form(
        xsd_string, data_structure=data_structure, request=request
    )
    # get the root element
    root_element = data_structure_element_api.get_by_id(
        root_element_id, request=request
    )

    return root_element


def render_form(request, root_element):
    """Renders the form

    Args:
        request:
        root_element:

    Returns:

    """
    # build a renderer
    renderer = CustomCheckboxRenderer(root_element, request)
    # render the form
    xsd_form = renderer.render()

    return xsd_form


# TODO: need to be reworked + similar as code in curate app
def remove_form_element(request, element_id):
    """Remove an element from the form.

    Args:
        request:
        element_id:

    Returns:

    """
    # Removing the element from the data structure
    data_structure_element_to_pull = data_structure_element_api.get_by_id(
        element_id, request
    )
    data_structure_element = data_structure_element_to_pull.parent

    # number of children after deletion
    children_number = data_structure_element.children.count() - 1

    data_structure_element = remove_child_element(
        data_structure_element, data_structure_element_to_pull, request
    )

    code = 0
    html_form = ""

    if children_number <= data_structure_element.options["min"]:
        # len(schema_element.children) == schema_element.options['min']
        if data_structure_element.options["min"] != 0:
            code = 1
        else:  # schema_element.options['min'] == 0
            code = 2
            renderer = CustomCheckboxRenderer(data_structure_element, request)
            html_form = renderer.render(True)

    return code, html_form



""" Custom admin site for the Explore Data Structure model
"""
from django.contrib import admin


class CustomExploreDataStructureAdmin(admin.ModelAdmin):
    """CustomExploreDataStructureAdmin"""

    readonly_fields = ["template", "data_structure_element_root"]
    exclude = ["selected_fields_html_tree"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Data Structures"""
        return False



""" REST Views for Saved Queries
"""

from django.http import Http404
from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.commons import exceptions
import core_explore_example_app.components.saved_query.api as saved_query_api
from core_explore_example_app.rest.saved_query.serializers import SavedQuerySerializer


class SavedQueryList(APIView):
    """Get all SavedQuery"""

    permission_classes = (IsAuthenticated,)

    def get(self, request):
        """Get all SavedQuery, can be filtered by owner and template

        Url Parameters:

            template: template_id
            user_id: document_title

        Examples:

            ../saved/query/
            ../saved/query/?user_id=[id]
            ../saved/query/?template_id=[id]
            ../saved/query/?user_id=[id]&template_id=[id]

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of saved queries
            - code: 500
              content: Internal server error
        """
        try:
            saved_query_list = saved_query_api.get_all()

            # Apply filters
            user_id = self.request.query_params.get("user_id", None)
            if user_id is not None:
                saved_query_list = saved_query_list.filter(user_id=user_id)
            template_id = self.request.query_params.get("template_id", None)
            if template_id is not None:
                saved_query_list = saved_query_list.filter(template=str(template_id))

            # Serialize object
            return_value = SavedQuerySerializer(saved_query_list, many=True)
            # Return response
            return Response(return_value.data)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class SavedQueryDetail(APIView):
    """ " Get an SavedQuery"""

    permission_classes = (IsAuthenticated,)

    def get_object(self, pk):
        """Get SavedQuery from db

        Args:

            pk: ObjectId

        Returns:

            SavedQuery
        """
        try:
            return saved_query_api.get_by_id(pk)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """Retrieve a SavedQuery

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: SavedQuery
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            saved_query_object = self.get_object(pk)
            # Serialize object
            return_value = SavedQuerySerializer(saved_query_object)
            # Return response
            return Response(return_value.data)
        except Http404:
            content = {"message": "Saved Query not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def delete(self, request, pk):
        """Delete a SavedQuery

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 204
              content: Deletion succeed
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            saved_query_object = self.get_object(pk)
            # if the user is owner or super user
            if (
                str(request.user.id) != saved_query_object.user_id
                and not request.user.is_staff
            ):
                return Response(status=status.HTTP_403_FORBIDDEN)
            # Remove the saved query
            saved_query_api.delete(saved_query_object)
            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except Http404:
            content = {"message": "Saved Query not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



""" Rights for core explore example app
"""
EXPLORE_EXAMPLE_CONTENT_TYPE = "core_explore_example_app"
EXPLORE_EXAMPLE_ACCESS = "access_explore_example"
EXPLORE_EXAMPLE_SAVE_QUERY = "save_query"
EXPLORE_EXAMPLE_DELETE_QUERY = "delete_query"
EXPLORE_EXAMPLE_DATA_STRUCTURE_ACCESS = "access_explore_example_data_structure"



""" Apps file for setting core package when app is ready
"""
import sys

from django.apps import AppConfig

from core_explore_example_app.permissions import discover


class ExploreExampleAppConfig(AppConfig):
    """Core application settings"""

    name = "core_explore_example_app"
    verbose_name = "Core Explore by Example App"

    def ready(self):
        """Runs when the app is ready

        Returns:

        """
        if "migrate" not in sys.argv:
            from core_explore_example_app import discover as app_discover

            app_discover.init_periodic_tasks()
            discover.init_permissions(self.apps)



""" Explore Common App tasks
"""
import logging
from datetime import timedelta

from celery import shared_task
from django.utils import timezone

from core_explore_example_app.settings import QUERIES_MAX_DAYS_IN_DATABASE
from core_explore_example_app.system.api import get_saved_queries_created_by_app

logger = logging.getLogger(__name__)


@shared_task
def delete_temporary_saved_queries():
    """Every day at midnight, delete older temporary saved queries.

    Returns:

    """
    try:
        # get older queries
        old_queries = [
            query
            for query in get_saved_queries_created_by_app()
            if query.creation_date
            < timezone.now() - timedelta(days=QUERIES_MAX_DAYS_IN_DATABASE)
        ]
        # remove old queries from database
        for query in old_queries:
            logger.info("Periodic task: delete saved query %s.", str(query.id))
            query.delete()
    except Exception as exception:
        logger.error(
            "An error occurred while deleting temporary saved queries (%s).",
            str(exception),
        )



"""Custom Checkbox Renderer class
"""
from core_parser_app.tools.parser.renderer.checkbox import CheckboxRenderer


class CustomCheckboxRenderer(CheckboxRenderer):
    """Custom Checkbox renderer, makes elements selectable and allows only one occurrence of each element"""

    def render_element(self, element):
        """render_element
        Args:
            element:

        Returns:
        """
        return super().render_element(_restrict_occurs(element))

    def render_attribute(self, element):
        """render_attribute
        Args:
            element:

        Returns:
        """
        return super().render_attribute(_restrict_occurs(element))

    def render_sequence(self, element, force_full_display=False):
        """render_sequence
        Args:
            element:
            force_full_display:

        Returns:
        """
        return super().render_sequence(
            _restrict_occurs(element), force_full_display=force_full_display
        )

    def render_choice(self, element):
        """render_choice
        Args:
            element:

        Returns:
        """
        return super().render_choice(_restrict_occurs(element))


def _restrict_occurs(element):
    """Restrict the number of occurrences to 1

    Args:
        element:

    Returns:

    """
    # if min number of occurrences is set
    if "min" in element.options:
        # if more than one occurrence
        if element.options["min"] > 1:
            # keep only first element of the list
            element.children.set([element.children.all()[0]])
            # force min occurrences to one
            element.options["min"] = 1
    # force max number of occurrences to one
    element.options["max"] = 1
    # return element
    return element



""" Auto discovery of explore example app.
"""
import logging

from django.core.exceptions import ObjectDoesNotExist
from django_celery_beat.models import CrontabSchedule, PeriodicTask

from core_explore_example_app.tasks import delete_temporary_saved_queries

logger = logging.getLogger(__name__)


def init_periodic_tasks():
    """Create periodic tasks for the app and add them to a crontab schedule"""
    try:
        schedule, _ = CrontabSchedule.objects.get_or_create(
            hour=0,
            minute=0,
        )
        PeriodicTask.objects.get(name=delete_temporary_saved_queries.__name__)
    except ObjectDoesNotExist:
        PeriodicTask.objects.create(
            crontab=schedule,
            name=delete_temporary_saved_queries.__name__,
            task="core_explore_example_app.tasks.delete_temporary_saved_queries",
        )
    except Exception as exception:
        logger.error(str(exception))



""" Fixtures files for Persistent Query Example
"""
from core_main_app.components.template.models import Template
from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface
from core_explore_example_app.components.persistent_query_example.models import (
    PersistentQueryExample,
)


class PersistentQueryExampleFixtures(FixtureInterface):
    """Persistent query example fixtures"""

    template = None
    persistent_query_example_1 = None
    persistent_query_example_2 = None
    persistent_query_example_3 = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_query_collection()

    def generate_template(self):
        """generate_template"""

        self.template = Template(filename="filename", content="<xml />", hash="hash")
        self.template.save()

    def generate_query_collection(self):
        """Generate a Persistent query example collection.

        Returns:

        """

        # NOTE: no xml_content to avoid using unsupported GridFS mock
        self.persistent_query_example_1 = PersistentQueryExample(
            user_id="1", name="persistent_query_example_1"
        )
        self.persistent_query_example_1.save()

        self.persistent_query_example_2 = PersistentQueryExample(
            user_id="2", name="persistent_query_example_2"
        )
        self.persistent_query_example_2.save()

        self.persistent_query_example_3 = PersistentQueryExample(
            user_id="3", name="persistent_query_example_3"
        )
        self.persistent_query_example_3.save()

        self.data_collection = [
            self.persistent_query_example_1,
            self.persistent_query_example_2,
            self.persistent_query_example_3,
        ]
